//  ************* TinyFlea microcomputer framework ************
//  ** For use with Fleatiny system board revision 1.2+ PCBs **
//  ** or compatible 8051-based hardware clones              **
//  **************** Kernel build # 0051 **********************

// Note: In unmodified form, this code works on 40/44-pin variants of 
// STC12Cxxxx (or STC12LExxxx) series of 8052 microcontrollers. However, 
// this framework should work with many models of *fast/modern 8052 micro 
// by modifying videotext driver module as well as small modifications to 
// video timer values and SFR initialisation. Also note that at least 1K of
// XRAM or more is required to allow for the VGA text or SD card buffers.
//(*fast/modern meaning running at 10 million instructions/sec. or more)


//  Copyright(c) 2012 Valentin Angelovski

//  This file is part of my Fleatiny embedded framework. Please refer 
//  to <http://www.fleasystems.com/fleatiny.html> for further info.

//  Fleatiny embedded framework is free software: you can redistribute it and/or 
//  modify it under the terms of the GNU Lesser General Public License as published 
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.

//  Fleatiny embedded framework is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.

//   You should have received a copy of the GNU Lesser General Public License
//   along with Fleatiny embedded framework.  If not, see <http://www.gnu.org/licenses/>.


// *** Fleatiny Kernel system variables declaration:

volatile __data unsigned char    GOT_KBRAW = 0;     // PS/2 KBD LATCHED RAW DATA BYTE 
volatile __data unsigned char    newchar = 0;
volatile __data unsigned char    lcount = 0; 		// Line counter
volatile __data unsigned char    vpl = 0;
volatile __data unsigned char    vph = 0;
volatile __data unsigned char    HLINES = 0;
volatile __data unsigned char    cursor_xpos = 0;
volatile __data unsigned char    cursor_ypos = 0; 
 
 
volatile __data unsigned char 	burst1 = 0;   // " " " " "
volatile __data unsigned char 	PS2_KEYC = 0; // Final received key character
volatile __data unsigned char 	KBD_OK = 0; // KEYBOARD DATA OK flag
volatile __data unsigned char 	BIT_COUNT2 = 0;

volatile __data unsigned char	opcode = 0;
volatile __data unsigned short	dataptr = 0;
volatile __data unsigned char 	tick_timer = 0;   // " " " " " 
 
 
volatile __data unsigned char    BIT_BASH2 = 0;
volatile __data unsigned char    BIT_BYTE2 = 0;
volatile __data unsigned char    KBD_OV = 0;
volatile __data unsigned char    TSTK1 = 0;
volatile __data unsigned char    TSTK2 = 0;
 
volatile __data unsigned char    BK_ACC = 0;
volatile __data unsigned char    BK_PSW = 0;
volatile __data unsigned char    BK_B = 0;
volatile __data unsigned char    BK_DPL = 0;
volatile __data unsigned char    BK_DPH = 0;

volatile __data unsigned char	BIT_COUNT = 0;
volatile __data unsigned char	BIT_BASH = 0;
volatile __data unsigned char	BIT_BYTE = 0;


volatile __bit	async_en = 0;
volatile __bit	async_run = 0;
volatile __bit	key_shifted = 0;
volatile __bit	key_break = 0;
volatile __bit	cursor_enable = 1; 	//  Text cursor enable flag
volatile __bit	cga_ready = 1;  	//  CGA vertical retrace flag set

volatile __bit	MOUSE_ER = 0; 		// PS/2 PROTOCOL ERROR
volatile __bit	PS2_GOTK = 0;  		// 'Got Key' flag
volatile __bit	vpixel_end = 0;		// vertical pixel end :-)
 
// *** PS/2 keyboard Scancode set #2 --> #1 translation table
const unsigned char bios_scancode[] = {
/*0 1  2 3  4  5  6  7  8  9 */
0, 67 ,0 ,63 ,61 ,59 ,60 ,88 ,0 ,68 ,66 ,64 ,62 ,15 ,
41 ,0 ,0 ,56 ,42 ,0 ,29 ,16 ,2 ,0 ,0 ,0 ,44 ,31 ,30 ,
17 ,3 ,91 ,0 ,46 ,45 ,32 ,18 ,5 ,4 ,92 ,0 ,57 ,47 ,33 ,
20 ,19 ,6 ,93 ,0 ,49 ,48 ,35 ,34 ,21 ,7 ,0 ,0 ,0 ,50 ,
36 ,22 ,8 ,9 ,0 ,0 ,51 ,37 ,23 ,24 ,11 ,10 ,0 ,0 ,52 ,
53 ,38 ,39 ,25 ,12 ,0 ,0 ,0 ,40 ,0 ,26 ,13 ,0 ,0 ,58 ,
54 ,28 ,27 ,0 ,43 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,14 ,0 ,0 ,79 ,
0 ,75 ,71 ,0 ,0 ,0 ,82 ,83 ,80 ,76 ,77 ,72 ,1 ,69 ,87 ,78 ,
81 ,74 ,55 ,73 ,70 ,0 ,0 ,0 ,0 ,65 ,0 
};  


// *** Allocate some space for the for the VGA text RAM buffer
__xdata __at (0x0000) unsigned char VRAM[512];			// tinyflea Video buffer! :-)
__xdata __at (0x0000) unsigned char matrix_char[16][32];


// #include "kernel_SD.h" // Not used
#include "TinyBASIC.h" // Include user program header file(s) here (in this case TinyBASIC interpreter)
#include "stdlib.h"
#include "math.h"
#include "ctype.h"
#include "string.h"

#include <8051.h>
#include <fleatiny_pinout.h>
 
// #include "kernel_SD.c" // Not used

#include "kernel_VTTY.c" // Fleatiny kernel User Input/Output module (Handles VGA text and TTY io functions)
#include "TinyBASIC.c" // Include your user application here (in this case TinyBASIC interpreter)

 
// *** VGA H-PERIOD timer interrupt event handler
void Timer_0_Interrupt (void) __interrupt (1) __naked
{
  __asm
	MOV		TL0,#0xAA	; LOAD TIMER LSB FIRST... SO TIMER MAY STILL WORK ON-THE-FLY!
	MOV		TH0,#0xFF	; LOAD UPPER TIMER BYTE WITH LOW TIME MSB
  
	MOV		TL1,#0xF4   ; TIMER 1 INITIAL SEED VALUE
	MOV		TH1,#0xFF   ; TIMER 1 AUTO-RELOAD VALUE
 
	CLR		_async_en	;// Disable asynchronous task (BASIC interpreter)
	CLR		_async_run	;      
	RETI
  __endasm;
}

// *** VGA HPULSE / USER PROGRAM TIMESLICE timer interrupt event handler
void Timer_1_Interrupt (void) __interrupt (3) __naked
{
__asm
	JB		_async_en, TIMR1a ;// Asynchronous task request pending?
	SJMP	TIMR1f
   
TIMR1a:     
	MOV		TL1,#0xC8   ; TIMER 1 INITIAL SEED VALUE
	MOV		TH1,#0xFF   ; TIMER 1 AUTO-RELOAD VALUE
     
	MOV 	_TSTK1,SP  ;// Task Switch between kernel and asynchronous task
	MOV 	SP,_TSTK2  ; 
	MOV 	_TSTK2,_TSTK1 ;    
      
	JNB		_async_run, TIMR1b ;// Determine asynchronous task status
       
    MOV		_BK_ACC, A  ;// Save data to 'shadow' register bank
    MOV		_BK_PSW, PSW    ;    
	MOV		_BK_B, B  ;
	MOV		_BK_DPH, DPH    ;    
	MOV		_BK_DPL, DPL    ;  
  
	CLR 	_async_en  ;// Kernel routine has control of the MCU
	CLR 	_async_run  ;
  SJMP TIMR1e
        
        
        
TIMR1b:      ;// Restore data from 'shadow' register bank        
    MOV 	A, _BK_ACC  ;
    MOV 	PSW, _BK_PSW    ;    
	MOV 	B, _BK_B  ;
	MOV 	DPH, _BK_DPH    ;    
	MOV 	DPL, _BK_DPL ;   
	SETB 	_async_run ;// Asyncronous task routine has control of the MCU
	NOP
	NOP
	NOP
	NOP  
	SETB  P3.7 ;
	RETI
  
; End task switch routine
TIMR1e:   
	NOP
	CLR		TR1    ;//23
	RETI 
 
TIMR1f:
	NOP   
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP   
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP 
    CLR		TR1    ;// 23
    RETI 
        
  __endasm;
 
}
 
 

// *** Fleatiny kernel code begins here 
void main(void)
{
 
 __asm

;/*****	Configure MIDAS 3.0 GPIO Port 1 *****/
;/* Port1 = VGA control lines, Audio DAC and SD card */

;// Port1 mode setting msbit (P1M1)
	MOV		0x91,#0x41
;// port1 mode setting lsbit (P1M0)
	MOV		0x92,#0xBE

	MOV		P1,#0xFF

;// port2 mode setting lsbit (P2M1)
	MOV 	0x95,#0x00
;// Port2 mode setting msbit (P2M0)
	MOV		0x96,#0xFF
	MOV		P2,#0xF0
 
 
;// port3 mode setting lsbit (P3M1):   Bits 0-1 open-drain. Bits 2-3 input only. Bits 4-7 push-pull output.
	MOV		0xB1,#0b00001110 ;//#0x0F
; Port3 mode setting msbit (P3M0)  
;// Note! on the STC parts, if GPIO is configured as anything BUT Hi-Z input, 
;// peripheral features of the affected pins will be disabled
	MOV		0xB2,#0b11110001;//#0xF3
	SETB	P3.4
;// MOV	P3,#0xFF;//0x30
 
;// AUXR configuration - UART clock = sysclk / 2 and all ADC, SPI interrupts disabled
	MOV   0x8E,#0x20 
 
;########################################################## 
 

SYS_RESET:
 

   
; //****************** PS/2 KEYBOARD & MOUSE INIT *******************
 
	MOV		_BIT_COUNT, #0xFF
	MOV		_BIT_COUNT2, #0xFF
	MOV		_BIT_BYTE, #8
	MOV		_BIT_BYTE2, #8
	MOV		_PS2_KEYC, #0
	MOV		_BIT_BASH, #0
	MOV		_BIT_BASH2, #0
	CLR		_PS2_GOTK
   
	SETB	IT0 ; SET PS2 KEYBOARD SCLK
	SETB	IT1 ; SET PS2 MOUSE SCLK 
    
; //*****************************************************************
 

    
    MOV		TMOD,#0x11 ; //PROGRAM TIMERS 1 & 2 FOR MODE 2
 
	MOV     TL0,#0xAA       ; LOAD TIMER LSB FIRST... SO TIMER MAY STILL WORK ON-THE-FLY!
	MOV		TH0,#0xFF       ; LOAD UPPER TIMER BYTE WITH LOW TIME MSB
  
	MOV		DPTR,#0000 ;CLEAR DATA POINTER
 
 
	MOV		TL1,#0xC0 ; TIMER 1 INITIAL SEED VALUE
	MOV		TH1,#0xFF ; TIMER 1 AUTO-RELOAD VALUE
 
	MOV		_TSTK2,#0xA1   ; //Setup stack pointer for asynchronous task
	mov 	r0,#0xA0
	MOV  	@r0,#_userprg    ; //Setup boot vector of asynchronous task (simple editor)
	inc 	r0
	MOV  	@r0,#(_userprg >> 8) ;
 
	MOV		SCON,#0x00 ;//UART 0 MODE 0 - SHIFT REGISTER for VGA Video use!!!! :D
	
	SETB  	ET0  ;ENABLE VGA VIDEO TIMER INTERRUPT
	SETB  	TR0  ;START VGA VIDEO TIMER
         
	SETB  	ET1  ;ENABLE INTERRUPT FOR THIS TIMER 
	CLR   	TR1  ;START FREE-RUNNING TIMER FOR RANDOM SEED        
	CLR   	ES
	SETB  	EA
 
 
 
 
MAIN2:

 SETB _cga_ready 
 ORL PCON,#1 ;
 NOP ;
 NOP
 SETB _async_en ; // RUN Asynchronous task (i.e. PC Emulator) 
 CLR P3.7 ;
 CLR P3.6 ;
 LCALL HDELAY ; PROCESS VSYNC - 1st 32 usecs
 SETB P3.7 ;
 
         
 ORL PCON,#1 ; enter idle mode.
 NOP ;
 NOP ;
 SETB _async_en ; // RUN Asynchronous task (i.e. PC Emulator)
 CLR P3.7 ;
 NOP
 LCALL HDELAY 	; //"    " - next 32 usecs = 64 usecs total
 SETB P3.7 ;
 
 
 ORL PCON,#1 ; enter idle mode.         
 NOP ;
 NOP
 SETB _async_en ; // RUN Asynchronous task (i.e. PC Emulator)   
 CLR P3.7 ;
 SETB P3.6 ; vsync completed
 LCALL HDELAY ;
 SETB P3.7 ;
 CLR  _cga_ready
 
 ; ***************** VGA timing engine part II - HSYNC timing *******************
 ; * 70 x 18 text resolution or 440 x 250 in graphics mode *
 ; ******************************************************************************
 
 MOV _HLINES,#84 ; load with start-blank lines to process...
 
 
MAIN3:
 
 ORL  PCON,#1 ; enter idle mode.   
 NOP  
 NOP  
 NOP  
 CLR  P3.7 ;
 SETB _async_en ; // RUN Asynchronous task (i.e. PC Emulator)
 LCALL HDELAY ;
 SETB P3.7 ; 
 
 DJNZ  _HLINES,MAIN3 ;
 
 
 ORL PCON,#1 ; enter idle mode.
 NOP ;
 NOP ;
 NOP
 CLR P3.7 ;
 NOP ; "    " - next 32 usecs = 64 usecs total
 LCALL HDELAY ;
 SETB P3.7 ;  
 
; ****************************
 ; issue VRAM address counter reset 
 ; ****************************
 
 
MAIN3b:
 
 LCALL R_VPTR ; reset video RAM address pointer 
 MOV  _HLINES,#128 ; load with number of VRAM lines to process...

 
MAIN4:
 
 ORL  PCON,#1 ; enter idle mode.
 NOP  
 NOP  
 NOP   
 
 CLR  P3.7 ;
 NOP  
 LCALL  HDELAY ;
 SETB  P3.7 ;
 NOP    
 NOP 
 NOP    
 NOP
 NOP    
 NOP 
 NOP    
 NOP
 CLR	_vpixel_end
 LCALL  _TEXTMODE
 
 ORL  PCON,#1 ; enter idle mode.
 NOP  
 NOP  
 NOP   
 
 CLR  P3.7 ;
 NOP  
 LCALL  HDELAY ;
 SETB  P3.7 ;
 DEC	_burst1
 ANL	_burst1,#0x07
 SETB	_vpixel_end
 LCALL  _TEXTMODE  
 ORL  PCON,#1 ; enter idle mode.
 NOP  
 NOP  
 NOP
 
 CLR  P3.7 ;
 SETB _async_en ;   // RUN Asynchronous task (i.e. PC Emulator) 
 LCALL  HDELAY ;
 SETB  P3.7 ;
 
 DJNZ  _HLINES,MAIN4 ;
 

MAIN5:
 ORL  PCON,#1 ; enter idle mode.
 NOP  
 NOP  
 NOP
 CLR  P3.7 ;
 SETB _async_en ; // RUN Asynchronous task (i.e. User program)
 LCALL HDELAY ;
 SETB P3.7 ;   
 
 MOV  _HLINES,#54 ; load with end-blank lines to process...
 
MAIN15:
 ORL  PCON,#1 ; enter idle mode.
 NOP  
 NOP
 NOP 
 
 CLR  P3.7 ;
 SETB _async_en ; // RUN Asynchronous task (i.e. User program)
 LCALL HDELAY ;
 SETB P3.7 ;   
 
 DJNZ _HLINES,MAIN15 ; 
 
 LJMP  MAIN2 
 
 

 
 
 ; ******* HORIZONAL SYNCRONISATION PERIOD - AUDIO DAC, PS/2 MOUSE + KEYBOARD HANDLED HERE ********
 
HDELAY: 
   SETB  TR1     ;START HS PULSE TIMER
 
   MOV  C,P3.2    ; CAPTURE PS/2 KEYBOARD DATA PIN STATE
   MOV  PSW.5, C   ; 
   JBC  IE1, PS2_KBA  ; KEYBOARD DATA available? Process if YES
   DJNZ  _KBD_OV, HDLY2    ; MOUSE DATA timeout reached? skip if NO
   MOV  _BIT_COUNT2,#0  ;   RESET framing error flag
HDLY2:
 

   
; *********** MOUSE AND KEYBOARD DATA ALL PROCESSED - NOW IT'S THE DAC's TURN *******   
MSE_DONE:
 
 
 ORL PCON,#1     ; enter idle mode.
 
 NOP       ;
 NOP       ;
 NOP       ;
 NOP       ; 
 RET 
 
 
 
 
 

PS2_KBA:
 
; ##################################################################################
; ################### PS/2 KEYBOARD-TO-HOST Communication Routine ##################
; ##################################################################################
PS2_KBD:
    MOV  _KBD_OV,#44   ; RESET BUFFER RECEIVE TIMEOUT
 MOV  A,_BIT_COUNT2  ; RESTORE frame task state
 CJNE A,#0,PS2_KB1
 LJMP KBD_STRT   ; EXECUTE START BIT Task
 
PS2_KB1:
 CJNE A,#1,PS2_KB2
 LJMP KBD_DATA   ; EXECUTE 8-DATA BITS Task
PS2_KB2: 
 CJNE A,#2,PS2_KB3
 LJMP KBD_PRTY   ; EXECUTE ODD PARITY BIT Task
PS2_KB3: 
 CJNE A,#3,PS2_KB4
 LJMP KBD_STOP   ; EXECUTE STOP BIT Task
PS2_KB4:
 LJMP MSE_DONE   ; Task done 
 
 
 
 
; ***************** START BIT Task *******************
KBD_STRT:
 MOV  _KBD_OK,#1   ; SET framing error flag
 INC  _BIT_COUNT2    ; 
 JB  PSW.5, KB_ERR    ; WAS THE START BIT LOW? FLAG ERROR IF HIGH
 LJMP MSE_DONE    ; Task done
 
 
 
; ***************** 8 DATA BITS task *******************
KBD_DATA: 
 MOV  C,PSW.5     ; READ OUR CUSTOM USER FLAG INTO CARRY 
 MOV  A, _BIT_BASH2   ; CAPTURE ONE DATA BIT RECEIVED FROM KBD
 RRC  A      ; 
 MOV  _BIT_BASH2, A   ;
 DJNZ _BIT_BYTE2,KBD_DAT2  ; WAS ALL 8 BITS RECEIVED?
 
 MOV  _BIT_BYTE2,#8   ; RESET BIT COUNT FOR DATA BYTE 
 INC  _BIT_COUNT2    ; SAVE frame task state
KBD_DAT2:
 LJMP MSE_DONE    ; Task done
 
 
 
; ***************** ODD PARITY BIT Task ******************* 
KBD_PRTY:
 INC  _BIT_COUNT2   ; RUN NEXT frame task state
 MOV  A, _BIT_BASH2  ; LOAD RECIEVED BYTE
 
; Odd parity check:
 
 JB  PSW.5,KBD_PTY2   ; 
 JNB  P,KB_ERR3   ; Catch Parity error
 LJMP MSE_DONE   ; Task done 
 
; Even parity check:
KBD_PTY2: 
 JB  P,KB_ERR3   ; Catch Parity error  
 LJMP MSE_DONE   ; Task done 
 
 
 
; ***************** STOP BIT Task ******************* 
KBD_STOP:
 JNB  PSW.5, KB_ERR4    ; WAS THE STOP BIT HIGH?
KBD_STP2:
;  //// Perform data integrity check here ////
 MOV  A, _KBD_OK
 JB  ACC.0, KBD_STP3
 MOV  _BIT_BASH2, #0   ; 
 MOV  _BIT_COUNT2, #0   ; CLEAR frame task state 
 LJMP MSE_DONE    ; Task done
 
 
 

; ***************** FRAME ERROR handler ******************* 
; 
; Flag sets on.. 1.) wrong start/stop states 2.) incorrect parity
 
KB_ERR:
 MOV  _KBD_OK,#2   ; SET framing error flag
 LJMP KBD_STP2   ; Task done
 
KB_ERR4:
 MOV  _KBD_OK,#4   ; SET framing error flag
 SJMP KBD_STP2   ; 
 
KB_ERR3:
 MOV  _KBD_OK,#8   ; SET framing error flag
 LJMP KBD_STP2   ; Task done
 
 
KBD_STP3:
 
;// &&&&&&&&&&&&&&& CONVERT KEYBOARD SCAN CODE SET 2 TO CODE SET 1 &&&&&&&&&&&&&&&
 MOV A, _BIT_BASH2
 
 CJNE A,#0xF0,KBDSHFT7  ; // Trap scancode set 2 key release prefix 0xF0
 SETB _key_break
 
 
 SJMP KBDSHFTD
 
KBDSHFT7:
 
;// MOV  _PS2_KEYC, A   ;  ... The scancode to ASCII data translation MUST occur within interrupt 9!
 SETB _PS2_GOTK     ;
 
 
     ; // Convert to scan code set 1

 mov  dptr,#_bios_scancode
 movc a,@a+dptr


 
 
 CJNE A,#0x2A,noshift
 SETB _key_shifted
 CLR  _PS2_GOTK      
 JNB  _key_break,noshift
 CLR  _PS2_GOTK   
 CLR  _key_shifted
noshift: 
 
 CJNE A,#0x36,noshift2
 SETB _key_shifted
 CLR  _PS2_GOTK 
 JNB  _key_break,noshift2
 CLR  _key_shifted
 CLR  _PS2_GOTK 
noshift2:  
 
 JB  _key_shifted,uppcase
 LCALL XLAT_LC 
 SJMP shiftdone
uppcase: 
 LCALL XLAT_UC
shiftdone:
 
 JNB  _key_break, KBDSHFTA ; PROCESS ANY key PRESS 
 CLR  _key_break
 ORL  A,#0x80     ;// Set 'key up' scancode bit 7 (MSBit)
 
KBDSHFTA: 
 mov  _GOT_KBRAW,A 
 
;// SETB P3.6     ;// Put the keyboard into command hold mode (IF SINGLE-STEPPING, DISABLE THIS!!!)
 
 
;// MOV  A,_i8259_imr   ;// Get interrupt mask register value
;// JB  ACC.1,KBDSHFTD   ;// Is keyboard hardware interrupt enabled?
 
;// MOV  A,_i8259_irr   ;// Set relevant interrupt request bit
;// SETB ACC.1
;// MOV  _i8259_irr,A   ;// Set relevant interrupt request bit
 
KBDSHFTD:
 MOV  _BIT_COUNT2, #0   ; CLEAR frame task state 
 LJMP MSE_DONE    ; Task done
 
; ****************** PS/2 SCANCODE TRANSLATION *****************
;// Scan code set 2 (********NEW*********)
XLAT_UC:
 MOVC A, @A+PC
 RET
.DB   27
.DB   33
.DB   64
.DB   35
.DB   36
.DB   37
.DB   94
.DB   38
.DB   42
.DB   40
.DB   41
.DB   95
.DB   43
.DB   8
.DB   9
.DB   81
.DB   87
.DB   69
.DB   82
.DB   84
.DB   89
.DB   85
.DB   73
.DB   79
.DB   80
.DB   123
.DB   125
.DB   13
.DB   0
.DB   65
.DB   83
.DB   68
.DB   70
.DB   71
.DB   72
.DB   74
.DB   75
.DB   76
.DB   58
.DB   34
.DB   126
.DB   0
.DB   124
.DB   90
.DB   88
.DB   67
.DB   86
.DB   66
.DB   78
.DB   77
.DB   60
.DB   62
.DB   63
.DB   0
.DB   0
.DB   0
.DB   32
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
 
 
 
 
 
 
 
 
 
 
 
XLAT_LC:
 MOVC A, @A+PC
 RET
.DB   27
.DB   49
.DB   50
.DB   51
.DB   52
.DB   53
.DB   54
.DB   55
.DB   56
.DB   57
.DB   48
.DB   45
.DB   61
.DB   8
.DB   9
.DB   113
.DB   119
.DB   101
.DB   114
.DB   116
.DB   121
.DB   117
.DB   105
.DB   111
.DB   112
.DB   91
.DB   93
.DB   13
.DB   0
.DB   97
.DB   115
.DB   100
.DB   102
.DB   103
.DB   104
.DB   106
.DB   107
.DB   108
.DB   59
.DB   39
.DB   96
.DB   0
.DB   92
.DB   122
.DB   120
.DB   99
.DB   118
.DB   98
.DB   110
.DB   109
.DB   44
.DB   46
.DB   47
.DB   0
.DB   0
.DB   0
.DB   32
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
.DB   0
 
 
 
XLAT_KP:
 MOVC A, @A+PC
 RET
.DB   0x37
.DB   0x38
.DB   0x39
.DB   0x2D
.DB   0x34
.DB   0x35
.DB   0x36
.DB   0x2B
.DB   0x31
.DB   0x32
.DB   0x33
.DB   0x30
.DB   0x2E
.DB   0x2A
 


;// ##################### VGA VIDEO FRAME re-initialise function ##########################
 
R_VPTR: 
 
 MOV  	_vpl,#0
 MOV  	_vph,#0
 
 MOV  	_burst1,#0x00
 DEC	_tick_timer	// Update system tick timer every 16.6 milliseconds (i.e. after every V-Sync pulse)
 RET
 
 
  __endasm;
 
}
